package docex;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.StringReader;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map.Entry;
import java.util.Vector;



public class ArticleSelector {
	HashMap<String,Double> corpus_idf = new HashMap<String,Double>();

	public void initialize() {
		ObjectInputStream in;
		try {
			in = new ObjectInputStream(new FileInputStream("word_idf.jserial"));
			corpus_idf = (HashMap<String,Double>) in.readObject();
			System.out.println("ArticleSelector.initialize() done "+corpus_idf.size()+" words read");
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}

	
	/**
	 * Chose articles from source that are close to the given main_article
	 * @param main_article
	 * @return
	 */
	public LinkedList<Article> selectCloseArticles(LinkedList<Article> source, Article main_article) {
		LinkedList<Article> close_articles = new LinkedList<Article>();
		Statistics stats = new Statistics(false);
		BufferedReader br;

		/**
		 * Build word frequency Maps on all articles 
		 */
		/* First parse main article */
		br = new BufferedReader(new StringReader(main_article.text));
		stats.readCorpus(br, false); // create per-article stats (but count multiple occurrences) 
		for (Integer i : stats.id_to_word.keySet()) {
			main_article.word_frequencies.put(stats.id_to_word.get(i), stats.id_to_count.get(i));
		}
		main_article.tfidf = Tools.TFIDFfromWordFrequency(corpus_idf,main_article.word_frequencies);
		stats.resetStatistics();
		
		System.out.println("--- main article done ---");
		
		/* Then parse all other candidates from the source */
		for (Article article : source) {
//			System.out.println("Next article: "+article.paragraphs.firstElement().getText());
			br = new BufferedReader(new StringReader(article.text));
			stats.readCorpus(br, false); // create per-article stats
//			System.out.println("selectCloseArticles() stats size after reading article text is "+stats.id_to_count.size());
			for (Integer i : stats.id_to_word.keySet()) {
				article.word_frequencies.put(stats.id_to_word.get(i), stats.id_to_count.get(i));
			}
			article.tfidf = Tools.TFIDFfromWordFrequency(corpus_idf,article.word_frequencies);
			stats.resetStatistics();

			article.cosine_similarity = Tools.similarityFromTFIDFVectors(article,main_article);
//			System.out.println("Article similarity "+article.relevance+"\n");
		}
	
		Collections.sort(source);

		for (Article a : source) {
			if (a.getCosine_similarity() > 0.2) {
				Tools.appendToFile("related.txt", a.getXMLString());
				close_articles.add(a);
			}
		}

		
//		Tools.clearFile("top_100.txt");
//		for (int i=0; i<100; i++) {
//			Article a = source.removeLast();
//
//			Tools.appendToFile("top_100.txt", a.getXMLString());
			
//			Double d = a.cosine_similarity*1000;
//			Integer s = new Integer(d.intValue());
//			String score = s.toString();
//			Tools.writeFile("selected__"+i+"__"+score+"__"+a.id+".txt", a.getReadableParagraphs());
//			if (source.isEmpty()) {
//				System.out.println("less than hundred articles to dump ... stopping here");
//				break;
//			}
//		}
		
	
		return close_articles;
	}

	
}




