package docex;

import java.util.Collections;
import java.util.Iterator;
import java.util.ArrayList;

public class Distributions {

	public static void main(String[] args) {
		ArrayList<Double> diff_orig = new ArrayList<Double>();
		ArrayList<Double> diff = new ArrayList<Double>();
		
		// lespwa + IHS (lespwa seed)
		// second half is relevant
		// should trigger at 185
//		ArrayList<Double> val = TWO_PARTITIONS_260();
		
		
		// should trigger around 40-70 (but very early in the set)
		// most of the 360 are relevant
//		ArrayList<Double> val = SLOW_RISE_JUMP_360();
	
		// same as above (but higher values in avg since longer main article)
		ArrayList<Double> val = TWO_PARTITIONS_LONG_MAIN_360();
		
		
		// should trigger at 196
		// !! TODO !! no trigger (cut at value?)
//		ArrayList<Double> val = KERVIEL4_ARTICLES_IN_200();
		
		
		Iterator<Double> v_iter = val.iterator();
				
		
		/**
		 * Argument for window-size?
		 * Looking at data sets with size of 5k to 25k. Doing 1000 steps
		 * with a 0.1% window size yields 5-25 steps as a window. However 5
		 * is far too small. -> Using 10 as a minimum.  
		 */
		int steps = 20; // window size for averaging
		
		for (int i=0; i<val.size()-1; i++) {
			diff_orig.add(Distributions.avg_diff(i, val, steps));
			System.out.format("%d %.4f %.4f %.4f \n",
						i, 
						val.get(i)/10,
						(val.get(i+1)-val.get(i)),
						Distributions.avg_diff(i, val, steps));
		}
		
		diff.addAll(diff_orig);
		Collections.sort(diff,Collections.reverseOrder());
		
		Double top = diff.get(0);
		System.out.format("top=%.4f  half=%.4f\n",top,top/2);
		

		/**
		 * As a starting point chances are that in a rare case 1% of all articles are
		 * relevant. Then to define a visible "edge" the amount of "rising" articles
		 * that would build a "ramp" in front of the relevant ones would at most be of
		 * similar size. (difference between a trapezoid and a triangle) 
		 */
		
		/**
		 * Since there is no month (and finally we would look at whole years or decades)
		 * that is smaller than 5000 articles, the 1% rule would look back >50 articles.
		 * Since we are looking at data sets that are missing more than 95% of articles
		 * we can force a minimum look-back of 30 diff-values. (which is still conservative)
		 */
		int pcnt1_size = (diff.size()/100) - 1; // 1% of size as a position in the array
		if (pcnt1_size < 30) pcnt1_size = 30;  
		
		System.out.format("diff count is %d \n",diff.size());
		System.out.format("pcnt 1 size is %.4f at %d \n",diff.get(pcnt1_size),pcnt1_size);
//		for (int i=0; i<pcnt1_size; i++) {
//			System.out.format("diff[%d]=%.4f \n",i,diff.get(i));
//		}

		/**
		 * If the distribution has distinct edges the set of differences should
		 * have very few "high" values -> check 
		 */
		if (diff.get(pcnt1_size) < (top/2) ) {
			System.out.println("This should have distinct edges (1% criterion)");
		} else {
			System.out.println("No distinct edges (most probably)");
			System.exit(-1);
		}
		
		
		/**
		 * Walk over diffs and look for the maximum appearing on the far left side
		 */

		boolean was_rising = false;
		Double acc_diff;
		for(int i=1; i<diff_orig.size(); i++) {
			acc_diff = avg_diff(i,diff_orig,steps) * steps;
			System.out.format("avg-diff-diff %.4f \n",acc_diff);
			if (acc_diff > 0.0004) {
				was_rising = true;
			} 
			if ( (acc_diff < -0.0004) & (was_rising) ) {
				System.out.println("leftmost maximum is at "+i);
				break;
			}
		}
		

		
	}


	/**
	 * Get the n-step average difference 
	 * @param nth
	 * @param vals
	 * @return
	 */
	public static Double avg_diff(int nth, ArrayList<Double> vals, int steps) {
		if ((steps%2) != 0) {
			System.out.println("averaging steps must be even number");
			System.exit(-1);
		}
			
		// lower end case
		if (nth <= (steps/2)) {
			return ((vals.get(steps)-vals.get(0))/steps);
		} 
		
		// upper end case
		if (nth > vals.size()-((steps/2)+1)) {
			return (vals.get(vals.size()-1)-(vals.get(vals.size()-((steps/2)+1)))/steps);
		}
		
		return ((vals.get(nth+(steps/2))-vals.get(nth-(steps/2)))/steps);
	}
	

	
	
	
	

	public static ArrayList<Double> TWO_PARTITIONS_LONG_MAIN_360() {
		ArrayList<Double> lst = new ArrayList<Double>();
		lst.add(0.03);
		lst.add(0.04);
		lst.add(0.044);
		lst.add(0.047);
		lst.add(0.047);
		lst.add(0.048);
		lst.add(0.048);
		lst.add(0.049);
		lst.add(0.049);
		lst.add(0.049);
		lst.add(0.051);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.061);
		lst.add(0.062);
		lst.add(0.063);
		lst.add(0.064);
		lst.add(0.065);
		lst.add(0.065);
		lst.add(0.066);
		lst.add(0.069);
		lst.add(0.07);
		lst.add(0.073);
		lst.add(0.073);
		lst.add(0.074);
		lst.add(0.075);
		lst.add(0.083);
		lst.add(0.084);
		lst.add(0.085);
		lst.add(0.086);
		lst.add(0.087);
		lst.add(0.089);
		lst.add(0.09);
		lst.add(0.094);
		lst.add(0.094);
		lst.add(0.094);
		lst.add(0.095);
		lst.add(0.095);
		lst.add(0.095);
		lst.add(0.1);
		lst.add(0.1);
		lst.add(0.103);
		lst.add(0.104);
		lst.add(0.107);
		lst.add(0.107);
		lst.add(0.109);
		lst.add(0.111);
		lst.add(0.116);
		lst.add(0.116);
		lst.add(0.12);
		lst.add(0.123);
		lst.add(0.129);
		lst.add(0.131);
		lst.add(0.132);
		lst.add(0.146);
		lst.add(0.15);
		lst.add(0.155);
		lst.add(0.157);
		lst.add(0.168);
		lst.add(0.173);
		lst.add(0.174);
		lst.add(0.185);
		lst.add(0.187);
		lst.add(0.189);
		lst.add(0.19);
		lst.add(0.201);
		lst.add(0.201);
		lst.add(0.202);
		lst.add(0.204);
		lst.add(0.206);
		lst.add(0.207);
		lst.add(0.209);
		lst.add(0.214);
		lst.add(0.216);
		lst.add(0.217);
		lst.add(0.221);
		lst.add(0.221);
		lst.add(0.222);
		lst.add(0.222);
		lst.add(0.225);
		lst.add(0.226);
		lst.add(0.227);
		lst.add(0.228);
		lst.add(0.228);
		lst.add(0.232);
		lst.add(0.235);
		lst.add(0.235);
		lst.add(0.236);
		lst.add(0.245);
		lst.add(0.25);
		lst.add(0.25);
		lst.add(0.254);
		lst.add(0.258);
		lst.add(0.259);
		lst.add(0.261);
		lst.add(0.266);
		lst.add(0.272);
		lst.add(0.275);
		lst.add(0.276);
		lst.add(0.283);
		lst.add(0.284);
		lst.add(0.284);
		lst.add(0.285);
		lst.add(0.286);
		lst.add(0.296);
		lst.add(0.296);
		lst.add(0.297);
		lst.add(0.297);
		lst.add(0.297);
		lst.add(0.3);
		lst.add(0.302);
		lst.add(0.302);
		lst.add(0.303);
		lst.add(0.305);
		lst.add(0.305);
		lst.add(0.306);
		lst.add(0.306);
		lst.add(0.309);
		lst.add(0.31);
		lst.add(0.314);
		lst.add(0.315);
		lst.add(0.316);
		lst.add(0.318);
		lst.add(0.319);
		lst.add(0.319);
		lst.add(0.321);
		lst.add(0.322);
		lst.add(0.323);
		lst.add(0.324);
		lst.add(0.325);
		lst.add(0.326);
		lst.add(0.327);
		lst.add(0.329);
		lst.add(0.329);
		lst.add(0.331);
		lst.add(0.331);
		lst.add(0.333);
		lst.add(0.334);
		lst.add(0.335);
		lst.add(0.338);
		lst.add(0.339);
		lst.add(0.34);
		lst.add(0.34);
		lst.add(0.34);
		lst.add(0.34);
		lst.add(0.342);
		lst.add(0.342);
		lst.add(0.342);
		lst.add(0.342);
		lst.add(0.343);
		lst.add(0.344);
		lst.add(0.347);
		lst.add(0.348);
		lst.add(0.349);
		lst.add(0.351);
		lst.add(0.351);
		lst.add(0.351);
		lst.add(0.352);
		lst.add(0.352);
		lst.add(0.352);
		lst.add(0.352);
		lst.add(0.352);
		lst.add(0.353);
		lst.add(0.355);
		lst.add(0.355);
		lst.add(0.355);
		lst.add(0.355);
		lst.add(0.358);
		lst.add(0.363);
		lst.add(0.364);
		lst.add(0.365);
		lst.add(0.365);
		lst.add(0.366);
		lst.add(0.366);
		lst.add(0.366);
		lst.add(0.367);
		lst.add(0.368);
		lst.add(0.369);
		lst.add(0.37);
		lst.add(0.371);
		lst.add(0.371);
		lst.add(0.372);
		lst.add(0.374);
		lst.add(0.374);
		lst.add(0.375);
		lst.add(0.375);
		lst.add(0.375);
		lst.add(0.377);
		lst.add(0.378);
		lst.add(0.378);
		lst.add(0.38);
		lst.add(0.381);
		lst.add(0.383);
		lst.add(0.383);
		lst.add(0.385);
		lst.add(0.385);
		lst.add(0.385);
		lst.add(0.386);
		lst.add(0.387);
		lst.add(0.387);
		lst.add(0.387);
		lst.add(0.389);
		lst.add(0.389);
		lst.add(0.39);
		lst.add(0.391);
		lst.add(0.391);
		lst.add(0.392);
		lst.add(0.392);
		lst.add(0.392);
		lst.add(0.392);
		lst.add(0.394);
		lst.add(0.394);
		lst.add(0.394);
		lst.add(0.394);
		lst.add(0.394);
		lst.add(0.395);
		lst.add(0.395);
		lst.add(0.396);
		lst.add(0.396);
		lst.add(0.396);
		lst.add(0.396);
		lst.add(0.396);
		lst.add(0.397);
		lst.add(0.399);
		lst.add(0.401);
		lst.add(0.401);
		lst.add(0.402);
		lst.add(0.403);
		lst.add(0.404);
		lst.add(0.405);
		lst.add(0.405);
		lst.add(0.406);
		lst.add(0.407);
		lst.add(0.407);
		lst.add(0.407);
		lst.add(0.408);
		lst.add(0.408);
		lst.add(0.41);
		lst.add(0.412);
		lst.add(0.413);
		lst.add(0.413);
		lst.add(0.413);
		lst.add(0.414);
		lst.add(0.415);
		lst.add(0.415);
		lst.add(0.415);
		lst.add(0.416);
		lst.add(0.417);
		lst.add(0.418);
		lst.add(0.419);
		lst.add(0.42);
		lst.add(0.42);
		lst.add(0.421);
		lst.add(0.421);
		lst.add(0.421);
		lst.add(0.422);
		lst.add(0.422);
		lst.add(0.423);
		lst.add(0.423);
		lst.add(0.423);
		lst.add(0.423);
		lst.add(0.424);
		lst.add(0.424);
		lst.add(0.426);
		lst.add(0.426);
		lst.add(0.426);
		lst.add(0.426);
		lst.add(0.427);
		lst.add(0.427);
		lst.add(0.427);
		lst.add(0.428);
		lst.add(0.428);
		lst.add(0.429);
		lst.add(0.43);
		lst.add(0.43);
		lst.add(0.43);
		lst.add(0.432);
		lst.add(0.434);
		lst.add(0.434);
		lst.add(0.434);
		lst.add(0.434);
		lst.add(0.435);
		lst.add(0.435);
		lst.add(0.437);
		lst.add(0.437);
		lst.add(0.437);
		lst.add(0.437);
		lst.add(0.438);
		lst.add(0.438);
		lst.add(0.438);
		lst.add(0.438);
		lst.add(0.439);
		lst.add(0.44);
		lst.add(0.44);
		lst.add(0.442);
		lst.add(0.443);
		lst.add(0.444);
		lst.add(0.445);
		lst.add(0.446);
		lst.add(0.447);
		lst.add(0.448);
		lst.add(0.448);
		lst.add(0.449);
		lst.add(0.45);
		lst.add(0.451);
		lst.add(0.451);
		lst.add(0.453);
		lst.add(0.454);
		lst.add(0.456);
		lst.add(0.457);
		lst.add(0.458);
		lst.add(0.458);
		lst.add(0.458);
		lst.add(0.458);
		lst.add(0.461);
		lst.add(0.461);
		lst.add(0.462);
		lst.add(0.462);
		lst.add(0.462);
		lst.add(0.464);
		lst.add(0.464);
		lst.add(0.465);
		lst.add(0.466);
		lst.add(0.466);
		lst.add(0.468);
		lst.add(0.468);
		lst.add(0.468);
		lst.add(0.469);
		lst.add(0.472);
		lst.add(0.474);
		lst.add(0.475);
		lst.add(0.477);
		lst.add(0.48);
		lst.add(0.488);
		lst.add(0.491);
		lst.add(0.492);
		lst.add(0.492);
		lst.add(0.493);
		lst.add(0.494);
		lst.add(0.496);
		lst.add(0.499);
		lst.add(0.499);
		lst.add(0.5);
		lst.add(0.504);
		lst.add(0.506);
		lst.add(0.509);
		lst.add(0.51);
		lst.add(0.511);
		lst.add(0.511);
		lst.add(0.514);
		lst.add(0.518);
		lst.add(0.533);
		lst.add(0.535);
		lst.add(0.537);
		lst.add(0.569);
		lst.add(0.595);
		lst.add(0.786);
		lst.add(0.805);
		lst.add(0.805);
		return lst;
	}



	public static ArrayList<Double> TWO_PARTITIONS_260() {
		ArrayList<Double> lst = new ArrayList<Double>();
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.01);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.011);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.012);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.013);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.014);
		lst.add(0.015);
		lst.add(0.015);
		lst.add(0.015);
		lst.add(0.015);
		lst.add(0.015);
		lst.add(0.015);
		lst.add(0.015);
		lst.add(0.015);
		lst.add(0.015);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.016);
		lst.add(0.017);
		lst.add(0.017);
		lst.add(0.017);
		lst.add(0.017);
		lst.add(0.017);
		lst.add(0.017);
		lst.add(0.017);
		lst.add(0.017);
		lst.add(0.017);
		lst.add(0.017);
		lst.add(0.017);
		lst.add(0.017);
		lst.add(0.018);
		lst.add(0.018);
		lst.add(0.018);
		lst.add(0.018);
		lst.add(0.018);
		lst.add(0.018);
		lst.add(0.018);
		lst.add(0.018);
		lst.add(0.018);
		lst.add(0.019);
		lst.add(0.019);
		lst.add(0.019);
		lst.add(0.019);
		lst.add(0.019);
		lst.add(0.019);
		lst.add(0.019);
		lst.add(0.019);
		lst.add(0.019);
		lst.add(0.019);
		lst.add(0.02);
		lst.add(0.02);
		lst.add(0.02);
		lst.add(0.02);
		lst.add(0.021);
		lst.add(0.021);
		lst.add(0.021);
		lst.add(0.021);
		lst.add(0.022);
		lst.add(0.022);
		lst.add(0.023);
		lst.add(0.023);
		lst.add(0.024);
		lst.add(0.024);
		lst.add(0.024);
		lst.add(0.026);
		lst.add(0.026);
		lst.add(0.028);
		lst.add(0.028);
		lst.add(0.029);
		lst.add(0.03);
		lst.add(0.03);
		lst.add(0.03);
		lst.add(0.031);
		lst.add(0.031);
		lst.add(0.04);
		lst.add(0.04);
		lst.add(0.043);
		lst.add(0.101);
		lst.add(0.108);
		lst.add(0.115);
		lst.add(0.123);
		lst.add(0.147);
		lst.add(0.15);
		lst.add(0.152);
		lst.add(0.154);
		lst.add(0.154);
		lst.add(0.155);
		lst.add(0.16);
		lst.add(0.16);
		lst.add(0.16);
		lst.add(0.165);
		lst.add(0.166);
		lst.add(0.168);
		lst.add(0.171);
		lst.add(0.175);
		lst.add(0.175);
		lst.add(0.179);
		lst.add(0.189);
		lst.add(0.19);
		lst.add(0.194);
		lst.add(0.194);
		lst.add(0.2);
		lst.add(0.201);
		lst.add(0.203);
		lst.add(0.205);
		lst.add(0.207);
		lst.add(0.207);
		lst.add(0.208);
		lst.add(0.209);
		lst.add(0.227);
		lst.add(0.229);
		lst.add(0.233);
		lst.add(0.248);
		lst.add(0.253);
		lst.add(0.259);
		lst.add(0.26);
		lst.add(0.26);
		lst.add(0.265);
		lst.add(0.266);
		lst.add(0.273);
		lst.add(0.287);
		lst.add(0.294);
		lst.add(0.297);
		lst.add(0.297);
		lst.add(0.298);
		lst.add(0.299);
		lst.add(0.301);
		lst.add(0.302);
		lst.add(0.302);
		lst.add(0.305);
		lst.add(0.306);
		lst.add(0.306);
		lst.add(0.308);
		lst.add(0.329);
		lst.add(0.33);
		lst.add(0.351);
		lst.add(0.359);
		lst.add(0.36);
		lst.add(0.361);
		lst.add(0.362);
		lst.add(0.362);
		lst.add(0.365);
		lst.add(0.383);
		lst.add(0.398);
		lst.add(0.412);
		lst.add(0.444);
		lst.add(0.462);
		lst.add(0.466);
		lst.add(0.541);
		lst.add(0.542);
		lst.add(1.00);
		return lst;
	}



	public static ArrayList<Double> SLOW_RISE_JUMP_360() {
		ArrayList<Double> lst = new ArrayList<Double>();
		lst.add(0.026);
		lst.add(0.031);
		lst.add(0.033);
		lst.add(0.034);
		lst.add(0.037);
		lst.add(0.037);
		lst.add(0.037);
		lst.add(0.037);
		lst.add(0.038);
		lst.add(0.039);
		lst.add(0.039);
		lst.add(0.039);
		lst.add(0.039);
		lst.add(0.04);
		lst.add(0.041);
		lst.add(0.042);
		lst.add(0.044);
		lst.add(0.045);
		lst.add(0.045);
		lst.add(0.046);
		lst.add(0.049);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.054);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.056);
		lst.add(0.058);
		lst.add(0.059);
		lst.add(0.06);
		lst.add(0.063);
		lst.add(0.066);
		lst.add(0.068);
		lst.add(0.076);
		lst.add(0.077);
		lst.add(0.078);
		lst.add(0.078);
		lst.add(0.078);
		lst.add(0.08);
		lst.add(0.081);
		lst.add(0.086);
		lst.add(0.086);
		lst.add(0.088);
		lst.add(0.09);
		lst.add(0.093);
		lst.add(0.093);
		lst.add(0.094);
		lst.add(0.097);
		lst.add(0.097);
		lst.add(0.102);
		lst.add(0.105);
		lst.add(0.105);
		lst.add(0.107);
		lst.add(0.108);
		lst.add(0.117);
		lst.add(0.117);
		lst.add(0.118);
		lst.add(0.133);
		lst.add(0.135);
		lst.add(0.136);
		lst.add(0.136);
		lst.add(0.137);
		lst.add(0.139);
		lst.add(0.14);
		lst.add(0.141);
		lst.add(0.142);
		lst.add(0.142);
		lst.add(0.143);
		lst.add(0.144);
		lst.add(0.145);
		lst.add(0.145);
		lst.add(0.147);
		lst.add(0.152);
		lst.add(0.154);
		lst.add(0.154);
		lst.add(0.155);
		lst.add(0.156);
		lst.add(0.159);
		lst.add(0.16);
		lst.add(0.161);
		lst.add(0.162);
		lst.add(0.162);
		lst.add(0.163);
		lst.add(0.165);
		lst.add(0.165);
		lst.add(0.166);
		lst.add(0.167);
		lst.add(0.168);
		lst.add(0.169);
		lst.add(0.17);
		lst.add(0.171);
		lst.add(0.172);
		lst.add(0.173);
		lst.add(0.173);
		lst.add(0.173);
		lst.add(0.174);
		lst.add(0.174);
		lst.add(0.175);
		lst.add(0.175);
		lst.add(0.177);
		lst.add(0.177);
		lst.add(0.177);
		lst.add(0.178);
		lst.add(0.179);
		lst.add(0.179);
		lst.add(0.18);
		lst.add(0.181);
		lst.add(0.181);
		lst.add(0.182);
		lst.add(0.184);
		lst.add(0.184);
		lst.add(0.185);
		lst.add(0.186);
		lst.add(0.187);
		lst.add(0.188);
		lst.add(0.188);
		lst.add(0.188);
		lst.add(0.189);
		lst.add(0.19);
		lst.add(0.191);
		lst.add(0.192);
		lst.add(0.192);
		lst.add(0.192);
		lst.add(0.193);
		lst.add(0.194);
		lst.add(0.197);
		lst.add(0.197);
		lst.add(0.197);
		lst.add(0.197);
		lst.add(0.198);
		lst.add(0.198);
		lst.add(0.198);
		lst.add(0.199);
		lst.add(0.199);
		lst.add(0.199);
		lst.add(0.199);
		lst.add(0.2);
		lst.add(0.2);
		lst.add(0.201);
		lst.add(0.201);
		lst.add(0.201);
		lst.add(0.202);
		lst.add(0.202);
		lst.add(0.203);
		lst.add(0.204);
		lst.add(0.204);
		lst.add(0.204);
		lst.add(0.205);
		lst.add(0.206);
		lst.add(0.206);
		lst.add(0.206);
		lst.add(0.207);
		lst.add(0.207);
		lst.add(0.207);
		lst.add(0.208);
		lst.add(0.208);
		lst.add(0.21);
		lst.add(0.21);
		lst.add(0.211);
		lst.add(0.212);
		lst.add(0.212);
		lst.add(0.212);
		lst.add(0.214);
		lst.add(0.214);
		lst.add(0.214);
		lst.add(0.215);
		lst.add(0.215);
		lst.add(0.216);
		lst.add(0.216);
		lst.add(0.216);
		lst.add(0.217);
		lst.add(0.217);
		lst.add(0.217);
		lst.add(0.217);
		lst.add(0.218);
		lst.add(0.219);
		lst.add(0.22);
		lst.add(0.22);
		lst.add(0.22);
		lst.add(0.22);
		lst.add(0.22);
		lst.add(0.22);
		lst.add(0.221);
		lst.add(0.222);
		lst.add(0.222);
		lst.add(0.222);
		lst.add(0.223);
		lst.add(0.224);
		lst.add(0.224);
		lst.add(0.224);
		lst.add(0.224);
		lst.add(0.225);
		lst.add(0.226);
		lst.add(0.226);
		lst.add(0.227);
		lst.add(0.228);
		lst.add(0.228);
		lst.add(0.228);
		lst.add(0.228);
		lst.add(0.228);
		lst.add(0.228);
		lst.add(0.229);
		lst.add(0.229);
		lst.add(0.229);
		lst.add(0.229);
		lst.add(0.229);
		lst.add(0.23);
		lst.add(0.23);
		lst.add(0.23);
		lst.add(0.23);
		lst.add(0.231);
		lst.add(0.231);
		lst.add(0.231);
		lst.add(0.231);
		lst.add(0.231);
		lst.add(0.232);
		lst.add(0.232);
		lst.add(0.232);
		lst.add(0.233);
		lst.add(0.234);
		lst.add(0.234);
		lst.add(0.235);
		lst.add(0.235);
		lst.add(0.235);
		lst.add(0.235);
		lst.add(0.235);
		lst.add(0.235);
		lst.add(0.235);
		lst.add(0.236);
		lst.add(0.236);
		lst.add(0.237);
		lst.add(0.237);
		lst.add(0.238);
		lst.add(0.24);
		lst.add(0.24);
		lst.add(0.24);
		lst.add(0.241);
		lst.add(0.241);
		lst.add(0.241);
		lst.add(0.241);
		lst.add(0.242);
		lst.add(0.243);
		lst.add(0.243);
		lst.add(0.243);
		lst.add(0.244);
		lst.add(0.244);
		lst.add(0.244);
		lst.add(0.244);
		lst.add(0.244);
		lst.add(0.245);
		lst.add(0.245);
		lst.add(0.245);
		lst.add(0.245);
		lst.add(0.245);
		lst.add(0.245);
		lst.add(0.246);
		lst.add(0.246);
		lst.add(0.246);
		lst.add(0.247);
		lst.add(0.248);
		lst.add(0.248);
		lst.add(0.248);
		lst.add(0.249);
		lst.add(0.249);
		lst.add(0.25);
		lst.add(0.25);
		lst.add(0.25);
		lst.add(0.252);
		lst.add(0.252);
		lst.add(0.253);
		lst.add(0.253);
		lst.add(0.253);
		lst.add(0.253);
		lst.add(0.254);
		lst.add(0.254);
		lst.add(0.254);
		lst.add(0.255);
		lst.add(0.255);
		lst.add(0.255);
		lst.add(0.255);
		lst.add(0.255);
		lst.add(0.255);
		lst.add(0.255);
		lst.add(0.256);
		lst.add(0.257);
		lst.add(0.257);
		lst.add(0.257);
		lst.add(0.257);
		lst.add(0.257);
		lst.add(0.258);
		lst.add(0.259);
		lst.add(0.259);
		lst.add(0.259);
		lst.add(0.259);
		lst.add(0.26);
		lst.add(0.261);
		lst.add(0.261);
		lst.add(0.262);
		lst.add(0.262);
		lst.add(0.262);
		lst.add(0.263);
		lst.add(0.264);
		lst.add(0.264);
		lst.add(0.264);
		lst.add(0.264);
		lst.add(0.264);
		lst.add(0.266);
		lst.add(0.266);
		lst.add(0.267);
		lst.add(0.267);
		lst.add(0.267);
		lst.add(0.267);
		lst.add(0.269);
		lst.add(0.269);
		lst.add(0.27);
		lst.add(0.27);
		lst.add(0.271);
		lst.add(0.271);
		lst.add(0.272);
		lst.add(0.274);
		lst.add(0.275);
		lst.add(0.277);
		lst.add(0.278);
		lst.add(0.278);
		lst.add(0.28);
		lst.add(0.28);
		lst.add(0.282);
		lst.add(0.284);
		lst.add(0.284);
		lst.add(0.287);
		lst.add(0.288);
		lst.add(0.29);
		lst.add(0.291);
		lst.add(0.294);
		lst.add(0.295);
		lst.add(0.303);
		lst.add(0.305);
		lst.add(0.305);
		lst.add(0.308);
		lst.add(0.312);
		lst.add(0.313);
		lst.add(0.313);
		lst.add(0.314);
		lst.add(0.315);
		lst.add(0.318);
		lst.add(0.318);
		lst.add(0.322);
		lst.add(0.325);
		lst.add(0.331);
		lst.add(0.333);
		lst.add(0.334);
		lst.add(0.336);
		lst.add(0.367);
		lst.add(0.37);
		lst.add(0.37);
		lst.add(0.681);
		lst.add(0.681);
		return lst;
	}

	
	
	
	
	
	public static ArrayList<Double> KERVIEL4_ARTICLES_IN_200() {
		ArrayList<Double> lst = new ArrayList<Double>();
		lst.add(0.049);
		lst.add(0.049);
		lst.add(0.049);
		lst.add(0.049);
		lst.add(0.049);
		lst.add(0.049);
		lst.add(0.049);
		lst.add(0.049);
		lst.add(0.05);
		lst.add(0.05);
		lst.add(0.05);
		lst.add(0.05);
		lst.add(0.05);
		lst.add(0.05);
		lst.add(0.05);
		lst.add(0.05);
		lst.add(0.05);
		lst.add(0.05);
		lst.add(0.05);
		lst.add(0.05);
		lst.add(0.051);
		lst.add(0.051);
		lst.add(0.051);
		lst.add(0.051);
		lst.add(0.051);
		lst.add(0.051);
		lst.add(0.051);
		lst.add(0.051);
		lst.add(0.051);
		lst.add(0.051);
		lst.add(0.051);
		lst.add(0.052);
		lst.add(0.052);
		lst.add(0.052);
		lst.add(0.052);
		lst.add(0.052);
		lst.add(0.052);
		lst.add(0.052);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.053);
		lst.add(0.054);
		lst.add(0.054);
		lst.add(0.054);
		lst.add(0.054);
		lst.add(0.054);
		lst.add(0.054);
		lst.add(0.054);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.055);
		lst.add(0.056);
		lst.add(0.056);
		lst.add(0.056);
		lst.add(0.056);
		lst.add(0.056);
		lst.add(0.056);
		lst.add(0.056);
		lst.add(0.057);
		lst.add(0.057);
		lst.add(0.057);
		lst.add(0.057);
		lst.add(0.057);
		lst.add(0.057);
		lst.add(0.057);
		lst.add(0.058);
		lst.add(0.058);
		lst.add(0.058);
		lst.add(0.058);
		lst.add(0.058);
		lst.add(0.058);
		lst.add(0.058);
		lst.add(0.058);
		lst.add(0.058);
		lst.add(0.058);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.059);
		lst.add(0.06);
		lst.add(0.06);
		lst.add(0.06);
		lst.add(0.06);
		lst.add(0.06);
		lst.add(0.06);
		lst.add(0.06);
		lst.add(0.061);
		lst.add(0.061);
		lst.add(0.061);
		lst.add(0.061);
		lst.add(0.061);
		lst.add(0.061);
		lst.add(0.061);
		lst.add(0.061);
		lst.add(0.062);
		lst.add(0.062);
		lst.add(0.062);
		lst.add(0.062);
		lst.add(0.062);
		lst.add(0.063);
		lst.add(0.063);
		lst.add(0.063);
		lst.add(0.063);
		lst.add(0.063);
		lst.add(0.063);
		lst.add(0.063);
		lst.add(0.063);
		lst.add(0.064);
		lst.add(0.064);
		lst.add(0.064);
		lst.add(0.064);
		lst.add(0.064);
		lst.add(0.064);
		lst.add(0.064);
		lst.add(0.065);
		lst.add(0.065);
		lst.add(0.065);
		lst.add(0.065);
		lst.add(0.065);
		lst.add(0.066);
		lst.add(0.066);
		lst.add(0.066);
		lst.add(0.066);
		lst.add(0.066);
		lst.add(0.067);
		lst.add(0.067);
		lst.add(0.067);
		lst.add(0.067);
		lst.add(0.067);
		lst.add(0.068);
		lst.add(0.068);
		lst.add(0.068);
		lst.add(0.068);
		lst.add(0.068);
		lst.add(0.068);
		lst.add(0.068);
		lst.add(0.069);
		lst.add(0.069);
		lst.add(0.069);
		lst.add(0.069);
		lst.add(0.069);
		lst.add(0.07);
		lst.add(0.07);
		lst.add(0.07);
		lst.add(0.07);
		lst.add(0.071);
		lst.add(0.071);
		lst.add(0.071);
		lst.add(0.071);
		lst.add(0.071);
		lst.add(0.071);
		lst.add(0.071);
		lst.add(0.072);
		lst.add(0.072);
		lst.add(0.073);
		lst.add(0.078);
		lst.add(0.079);
		lst.add(0.083);
		lst.add(0.083);
		lst.add(0.084);
		lst.add(0.085);
		lst.add(0.085);
		lst.add(0.085);
		lst.add(0.09);
		lst.add(0.09);
		lst.add(0.094);
		lst.add(0.098);
		lst.add(0.098);
		lst.add(0.098);
		lst.add(0.1);
		lst.add(0.101);
		lst.add(0.114);
		lst.add(0.335);
		lst.add(0.339);
		lst.add(0.423);
		lst.add(0.426);
		return lst;
	}


	
	
	

	
}
