package docex;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedList;

//import opennlp.tools.sentdetect.SentenceDetectorME;
//import opennlp.tools.sentdetect.SentenceModel;

/** 
 * @author cschulte
 *
 */
public class Docex {

	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		StreamSearchTrie st = new StreamSearchTrie();
				
		
		/**
		 * Cleanup corpus (remove text segments and articles that are useless for our analysis)
		 */
		if ((args.length != 0) && (args[0].equals("clean"))) {
			st.add("TOP STORIES:",""); // also has headline="WORLD at 1000GMT"
			st.add("AP Weekly News Calendar",""); 
			st.add("Today in History","");  // obviously history (look-back)
			st.add("Scorecard","");
			st.add("scorecard","");
			st.add("U.S. Open Results","");
			st.add("kih-JAH'-nah","");
			st.add("AH'-boo","");
			st.add("mahk-MOOD'","");
			st.add("MOO'-lah","");
			st.add("moh-HAHM'-ed","");
			st.add("AK'-bar","");
			st.add("MOOK'-tahr","");
			st.add("sah-EED'","");
			st.add("fuh-LOO'-juh","");
			st.add("hurts-uh-goh-VEE'-nah","");
			st.add("SEHR'-gay","");
			st.add("DRY'-fuhs","");
			st.add("MLAH'-dich","");
			
			BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
			st.searchAndExtract(br, true);
			System.exit(0);
		}
		
		/**
		 * Generate statistics that will later be used while selecting close articles
		 */
		if ((args.length != 0) && (args[0].equals("stats"))) {
			System.out.println("Starting corpus statistics run...");
			Statistics stat = new Statistics(false);
			
			BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
			stat.readCorpus(br,true);
		
			stat.printStats();
			stat.saveIDF();
			System.exit(0);
		}


		/**
		 * Select close (similar) articles to the ones given as file-input (main article)
		 * @INPUT <TEXT> segments (cleaned and keywords-extracted / all-extracted format)
		 */
		if ((args.length == 2) && (args[0].equals("select"))) {
			try {
				Article main_article = new Article(Tools.readFile(args[1], Charset.defaultCharset()));

				System.out.println("splitting input articles");
				BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
				LinkedList<Article> source = Tools.splitStreamToArticles(br);
				ArticleSelector sel = new ArticleSelector();
				System.out.println("init selector");
				sel.initialize();
				
				System.out.println("select articles");
				LinkedList<Article> close_articles = sel.selectCloseArticles(source, main_article);

				System.out.println("Found "+close_articles.size()+" close articles, from "+source.size()+" source articles");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			System.exit(0);
		}


		/**
		 *  
		 *                #### normal use case ####		 
		 *   		 
		 * Parse related material, detect duplicates, optimize and generate output
		 */
		if ((args.length >= 2) && args[0].equals("news")) {
			System.out.println("Generating StoryVector from input stream ...");
			BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
			LinkedList<Article> source = Tools.splitStreamToArticles(br);

			Date d = new Date();
			SimpleDateFormat ft = new SimpleDateFormat ("yyyy-MM-dd"); 
			try { 
				d = ft.parse(args[1]); 
			} catch (ParseException e) { 
				System.out.println("Unparseable using " + ft); 
				System.exit(-1);
			}

			StoryVector sv = new StoryVector();
			sv.initialize();
			sv.parseStories(source,d);

			if (args.length == 4) { // baseline reference that selects articles by distance
				int num = Integer.parseInt(args[3]);
				sv.removeDuplicateArticlesBySimilarity(num);
			} else {  // "our" version by optimizing selected paragraphs and then making a cut (like 90% textually complete articles)
				sv.optimize();
			}

			//sv.evaluateByGroupDistance();
			
			sv.printHtml();
			
			System.exit(0);
		}

		
		
		
//		/**
//		 * 
//		 * ##################  FOR EVALUATION ONLY  ##################
//		 * 
//		 * Parse related material, detect duplicates, optimize and generate output
//		 */
//		if ((args.length >= 2) && args[0].equals("news")) {
//			System.out.println("Generating StoryVector from input stream ...");
//			BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
//			LinkedList<Article> source = Tools.splitStreamToArticles(br);
//			LinkedList<Article> source_copy = new LinkedList<Article>();
//
//			// set up a deep copy of articles
//			for (Article a : source) {
//				source_copy.add(new Article(a.orig_input_string));
//			}
//			
//			Date d = new Date();
//			SimpleDateFormat ft = new SimpleDateFormat ("yyyy-MM-dd"); 
//			try { 
//				d = ft.parse(args[1]); 
//			} catch (ParseException e) { 
//				System.out.println("Unparseable using " + ft); 
//				System.exit(-1);
//			}
//
//			StoryVector sv = new StoryVector();
//			sv.initialize();
//			
//			// set the threshold that is going to be evaluated
//			sv.setPARAGRAPH_SIMILARITY_THRESHOLD(0.7);
//			sv.parseStories(source,d);
//			sv.optimize(); // see what we would pick as a summary
//			LinkedList<String> summaryIDs = sv.getSummaryIDs();
////			sv.setSummaryArticleState();
////			sv.resetOptimizationState();
//
//			
//			// get a fresh set of the related articles and set the 
//			// summary articles picked in the first run
//			sv = new StoryVector();
//			sv.initialize();
//			// set the threshold AGAINST which the above one is to be evaluated
//			sv.setPARAGRAPH_SIMILARITY_THRESHOLD(0.5);
//			sv.parseStories(source_copy,d);
//			sv.setSummaryIDs(summaryIDs);
//			
//			// with the above summary back in place and the reference
//			// duplicate-groups we can now compare both thresholds results
//			sv.printHtml(); 
//			
//			System.exit(0);
//		}

		
		
		

		
		/**
		 * A workaround for evaluating extraction on the whole corpus instead of
		 * some keyword-pre-extracted set of articles.
		 * This simply strips the DOC-segments and makes the <HEADLINE> a parameter
		 * headline="" of the TEXT-segment (plus some other cleanups not done in the
		 * above "clean" step) 
		 */
		if ((args.length == 1) && (args[0].equals("a"))) {
			System.out.println("Extract everything into singular file (reformat)");
			BufferedReader bi = new BufferedReader(new InputStreamReader(System.in));
			st.searchAndExtract(bi, false);
			System.exit(0);
		}		
		

		if (args.length == 0) {

			System.out.println("Running keyword extraction into separate files ...");

			//		st.add("Maurice Hank Greenberg","Maurice Hank Greenberg");
			//		st.add("Maurice \"Hank\" Greenberg","Maurice Hank Greenberg");
			//		st.add("Hank Greenberg","Maurice Hank Greenberg");
			//		
			//		st.add("Latvia national Election","Latvia national Election");
			//		st.add("Latvia Election","Latvia national Election");
			//		st.add("National Election Latvia","Latvia national Election");
			//		
			//		st.add("Viareggio train","Viareggio train derailment");
			//		st.add("derailment Viareggio","Viareggio train derailment");
			//		st.add("derailment in Viareggio","Viareggio train derailment");
			//
			//		st.add("Stanford Group","Stanford Financial Group");
			//		st.add("Stanford Financial","Stanford Financial Group");

			st.add("David Rohde","");
			st.add("Jerome Kerviel","");
			st.add("Flash Crash","");
			st.add("Rick Sanchez","");
			st.add("Richard Wolffe","");
			st.add("Pape Diouf","");
			st.add("Henry Moore","");
			st.add("Beryl Bainbridge","");
			st.add("University of Colorado","");
			st.add("Ardoyne","");
			st.add("Renesas","");
			st.add("Timor Sea","");
			st.add("KBC Group","");	
			st.add("Lespwa","");		
			st.add("Takanobu Ito",""); 
			st.add("George Tiller","");
			st.add("Brian Bethune","");
			st.add("Thomas Vermaelen","");
			st.add("Jeff Tillotson","");
			st.add("IHS Global Insight","");
			st.add("Flight 447","");
			st.add("Allen Stanford","");
			st.add("Rahm Emanuel","");
			st.add("Brian Bethune","");
			st.add("Alicia Sacramone","");
			st.add("McClatchy","");
			st.add("Mike Zafirovski","");
			st.add("Peter Bethune","");
			st.add("Mia Farrow","");
			st.add("Elpida Memory","");
			st.add("Brad Fittler","");
			st.add("Eric Woerth","");
			st.add("Woody Allen","");
			st.add("Amanda Knox","");

			//		st.add("Federal Reserve","US Federal Reserve Bank");
			//
			//		st.add("US Federal Reserve","US Federal Reserve Bank");
			//		st.add("United States Federal Reserve","US Federal Reserve Bank");
			//
			//		st.add("US Federal Reserve Bank","US Federal Reserve Bank");
			//		st.add("United States Federal Reserve Bank","US Federal Reserve Bank");
			//
			//		st.add("Federal Reserve System","US Federal Reserve Bank");
			//		st.add("US Central Bank","US Federal Reserve Bank");


			BufferedReader bi = new BufferedReader(new InputStreamReader(System.in));
			st.searchAndExtract(bi, false);
			
			System.exit(0);
		}
		
		System.out.println("Parameter format incorrect ?");
	}

}
