package docex;

import gurobi.GRBVar;

import java.util.HashMap;

public class Paragraph implements TFIDF {
	private String p_text = new String();
	private boolean enabled = true;
	public Article parent;
	public int pnum;  // nth paragraph in article

	// for optimization with gurobi
	public Integer distinct_par_id;
	public GRBVar opt_var;
	
	public HashMap<String,Integer> word_frequencies = new HashMap<String,Integer>();
	public HashMap<String,Double> tfidf = new HashMap<String,Double>();
	public Double cosine_similarity = (double) 0;
	public Double dotproduct_similarity = (double)0;
	
	public HashMap<String,Paragraph> expansions = new HashMap<String,Paragraph>();	

	public String getParId() {
		String pid = new String();
		pid = parent.id.substring(8);
		pid += "_P";
		pid += pnum;
		return pid;
	}
	
	public Paragraph(Article a, int pn, String text) {
		p_text = text;
		parent = a;
		pnum = pn;
		distinct_par_id = 0;
	}
	

	public String getText() {
		return p_text;
	}

	public void setText(String s) {
		p_text = s;
	}
	

	
	/* TFIDF - Interface */
	@Override
	public HashMap<String, Integer> getWord_frequencies() {
		return word_frequencies;
	}

	@Override
	public void setWord_frequencies(HashMap<String, Integer> wf) {
		word_frequencies = wf;
	}

	@Override
	public HashMap<String, Double> getTfidf() {
		return tfidf;
	}

	@Override
	public void setTfidf(HashMap<String, Double> newtfidf) {
		tfidf = newtfidf;
	}

	@Override
	public Double getCosine_similarity() {
		return cosine_similarity;
	}

	@Override
	public void setCosine_similarity(Double cosim) {
		cosine_similarity = cosim;
	}

	@Override
	public Double getDotproduct_similarity() {
		return dotproduct_similarity;
	}
	
	@Override
	public void setDotproduct_similarity(Double dotsim) {
		dotproduct_similarity = dotsim;
	}

	public boolean isEnabled() {
		return enabled;
	}
	
	public void enable() { enabled = true; }
	public void disable() { enabled = false; }


	public String refname() {
		String refname = parent.id+"__par_"+pnum;
		return refname;
	}

}
