package docex;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;
import java.util.TreeSet;

public class Statistics {

	public HashMap<String,Integer> word_to_id = new HashMap<String,Integer>();
	public TreeMap<Integer,String> id_to_word = new TreeMap<Integer,String>();
	public TreeMap<Integer,Integer> id_to_count = new TreeMap<Integer,Integer>(); 
	public int next_word_id = 0;
	public int corpus_word_count = 0;
	public Integer article_count = new Integer(0);
	
	public TreeSet<String> per_article_words = new TreeSet<String>();
	boolean per_article;


	public Statistics(boolean read_from_file) {
		if (read_from_file) {
			ObjectInputStream in;
			try {
				in = new ObjectInputStream(new FileInputStream("word_to_id.jserial"));
				word_to_id = (HashMap<String,Integer>) in.readObject();

				in = new ObjectInputStream(new FileInputStream("id_to_word.jserial"));
				id_to_word = (TreeMap<Integer,String>) in.readObject();

				in = new ObjectInputStream(new FileInputStream("id_to_count.jserial"));
				id_to_count = (TreeMap<Integer,Integer>) in.readObject();

				in = new ObjectInputStream(new FileInputStream("article_count.jserial"));
				article_count = (Integer) in.readObject();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	
    /**
     * In case we find a tag, check if its TEXT, 
     * If yes then submit the per-article-word-list to the statistics
     * @param s
     */
    public void readTag(BufferedReader s) {
    	int c=0;
    	int tag_pos = 0;
    	int tag[] = new int[1024];
    	
    	/* read the complete tag */
    	do {
    		try {
				c = s.read();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				System.exit(-1);
			}
    		tag[tag_pos] = c;
    		tag_pos++;
    	} while ((c != '>') && (c >= 0) && (tag_pos < 1024));
		if (c < 0) { System.out.println("end of stream in the middle of XML-tag (no closing > found)!"); System.exit(-1); }

		if (!per_article) {
			return;
		}
		
    	/* found <TEXT> tag */
    	if ( 	  (tag[0] == 'T')
    			&&(tag[1] == 'E')
    			&&(tag[2] == 'X')
    			&&(tag[3] == 'T')) {
//    		System.out.println("found <TEXT>");
    		per_article_words.clear();
    		article_count++;
    	}

    	/* found closing </TEXT> tag */
    	if ( 	  (tag[0] == '/')
    			&&(tag[1] == 'T')
    			&&(tag[2] == 'E')
    			&&(tag[3] == 'X')
    			&&(tag[4] == 'T')) {
//    		System.out.println("found </TEXT>");
    		submitArticleWords();
    	}
    }
	
	
    public void submitArticleWords() {
		article_count++;
		for (String word : per_article_words.descendingSet()) { 
			// add every word in the article only once
			addWord(word);
		}
    }
    
	
    
    /**
	 * Read corpus to initialize statistical data
     * 
     * @param s Input stream
     * @param pa [per article] If TRUE dont count duplicates (just create a set of words that appear in the input stream).
     * If FALSE generate default statistics in (id_to_count, id_to_word, word_to_id)
     */
	public void readCorpus(BufferedReader s, boolean pa) {
    	int c=0;
    	String word = new String();
    	per_article = pa;
    	    	
    	try {
    		while(true) {
				c = s.read();
				if (c < 0) { // EOF (end of stream)
   					addWord(word);
   					word = "";
					return;
				}
				
    			if (c == '<') {
   					addWord(word);
   					word = "";
    				readTag(s);
    				continue;
    			}

    			c = Tools.toLower(c);

    			if (		(!Tools.isAlphaNumeric(c))
    					&&	(word.length() != 0) )
    			{
    				if (per_article) {
    					per_article_words.add(word);
    				} else {
    					addWord(word);
    				}
    				word = "";
    			} else if (!Tools.isAlphaNumeric(c)) {
    				/* potential start of word (delimiter) */
    				word = new String();
    			}

    			/**
    			 *  Append characters we would consecutively expect in a word.
    			 *  @TODO since the corpus does not contain hyphen-newline-separated words
    			 *  we can assume that all hyphens we see are ment to be (stay) there   
    			 */
				if (	Tools.isAlpha(c)
					|	((c=='-') & (!word.isEmpty()))  // ie. "Claus-Peter" not "-Claus-Peter" 
					) {
					word += (char)c;
				} else {  // otherwise reset word
					word = "";
				}
    		}
    	} catch (IOException e) {
    		// TODO Auto-generated catch block
    		e.printStackTrace();
    	}
	}

	
	/**
	 * Accept everything of lentgh 3 and longer if it is
	 * not a stop-word. Update stats (word frequency) in
	 * word_to_id
	 * id_to_word
	 * id_to_count
	 * @param word
	 */
	public void addWord(String word) {
		if (word.length()<3) {
			return;
		}
		
		corpus_word_count++;
		
		if (Tools.stopWords.contains(word))
			return; // we dont count you, go away

		/* check if word exists in tables (did we see it already)*/
		if (!word_to_id.containsKey(word)) {
//			System.out.println("new word "+word);
			word_to_id.put(word, next_word_id);
			id_to_word.put(next_word_id, word);
			id_to_count.put(next_word_id, 1);
			next_word_id++;
		} else {
			int index = word_to_id.get(word);
			id_to_count.put(index, id_to_count.get(index) + 1);
		}
	}
		
	
	
	public void printStats() {
		ArrayList<IdCountPair> sorted_by_counts = new ArrayList<IdCountPair>();

		for (Integer i : id_to_count.keySet()) {
			sorted_by_counts.add(new IdCountPair(i,id_to_count.get(i)));
		}
		
		Collections.sort(sorted_by_counts);

		System.out.println("\ncorpus word count is "+corpus_word_count);
		System.out.println("article count "+article_count);
		System.out.println("size of id_to_count is "+id_to_count.size());
		System.out.println("size of sorted_by_counts is "+sorted_by_counts.size());
		
		int some = id_to_word.size();
		if (some > 40) some = 40;
		for (int c=0; c<some; c++) {
			System.out.println(
					 id_to_word.get(sorted_by_counts.get(c).ID)
					+" -> "
					+sorted_by_counts.get(c).Count);
		}
	}

	
	/**
	 * Dump collected statistics into files
	 */
	public void saveStats() {
		System.out.println("dumping word frequencies");
		
		try {
			ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream("word_to_id.jserial"));
			out.writeObject(word_to_id);
			out.close();

			out = new ObjectOutputStream(new FileOutputStream("id_to_word.jserial"));
			out.writeObject(id_to_word);
			out.close();

			out = new ObjectOutputStream(new FileOutputStream("id_to_count.jserial"));
			out.writeObject(id_to_count);
			out.close();

			out = new ObjectOutputStream(new FileOutputStream("article_count.jserial"));
			out.writeObject(article_count);
			out.close();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}


	public void saveIDF() {
		HashMap<String,Double> idf = new HashMap<String,Double>();
		
		System.out.println("\ndumping word-IDF HashMap");
		
		for (int k : id_to_word.keySet()) {
			Double d = new Double(id_to_count.get(k));
			d = Math.log(article_count/d)/Math.log(2); // log_base_2(article_count/word_count)
			idf.put(id_to_word.get(k), d);
		}

		Iterator<String> iter = idf.keySet().iterator();

		int some = idf.size();
		if (some > 40) some = 40;
		for (int c=0; c<some; c++) {
			String word = iter.next();
			System.out.println(
					 word
					+" -> "
					+idf.get(word));
		}
		
		try {
			ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream("word_idf.jserial"));
			out.writeObject(idf);
			out.close();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}

	
	public void resetStatistics() {
		word_to_id.clear();
		id_to_word.clear();
		id_to_count = new TreeMap<Integer,Integer>(); 
		next_word_id = 0;
		corpus_word_count = 0;
		article_count = new Integer(0);
		per_article_words = new TreeSet<String>();
	}
	
}







