package docex;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

  

public class StreamSearchTrie {
	HashMap<String,String> doc_hmap = new HashMap<String,String>();
	final int MAX_DOC_SIZE = (1024*1024);
	int doc_segment[] = new int[MAX_DOC_SIZE];
	int doc_pos = 0;
	boolean cleanup = false; // inverse document extraction for cleaning up corpus

	boolean search_term_found = false;
	String search_term = ""; 
    TNode root;
 
    
    public StreamSearchTrie() {
        root = new TNode();
     } 

    
    /**
     *  Insert string in TRIE
     */
    public void add(String s, String canonical_s) {
        TNode curr_node = root;
        if (canonical_s == "")
        	canonical_s = s; // keep spelling of the search term (for canonical output)
        s = s.toLowerCase(); 
        for (int i = 0, n = s.length(); i < n; i++) {
            Character c = s.charAt(i);
            if (curr_node.hmap.containsKey(c))
                curr_node = curr_node.hmap.get(c);
            else {
                curr_node.hmap.put(c, new TNode());
                curr_node = curr_node.hmap.get(c);
            }
        }
        curr_node.isTerminal = true;
      	curr_node.term = canonical_s;
    }


    /**
     * Create/Append the found DOC snippet to a file with the first
     * search term we have found in the DOC section
     */
    public void appendDocSegmentToFile() {
    	int c;
    	FileOutputStream out = null;

    	try {
    		out = new FileOutputStream(search_term + ".extract", true);
    		for (c=0; c<doc_pos; c++) {
    			out.write(doc_segment[c]);
    		}
    		out.close();
    	} catch (IOException e) {
    		throw new RuntimeException(e);
    	}
    }

    
    
    /**
     * Create/Append the found DOC snippet to a file with the first
     * search term we have found in the DOC section
     */
    public void appendStringToFile(String s) {
    	try {
    	    BufferedWriter bwriter = new BufferedWriter(new FileWriter(search_term + ".extract", true));
    	    PrintWriter pwriter = new PrintWriter(bwriter);
    	    pwriter.write(s);
    	    pwriter.close();
    	} catch (IOException e) {
    		e.printStackTrace();
    	}
    }

    
    
    
    /**
     * trims the DOC-segment to a TEXT-segment while keeping the document-ID
     */
    public String docToText(String doc) {
    	String text = new String();
    	int start,end;
    	
    	start = doc.indexOf("id=\"") + 4;
    	text = "<TEXT id=\"" + doc.substring(start, start + 21) + "\""; // grab the whole ID-string
    	
    	start = doc.indexOf("<HEADLINE>") + 11;
    	end = doc.indexOf("</HEADLINE>") - 1;

    	if ((start > 0) && (end>0)) {
    		text += " headline=\"" + doc.substring(start,end) + "\""; 
    	}
    	
    	text +=">\n";
    	start = doc.indexOf("<TEXT>") + 7;

    	if (start <= 0) {
    		System.out.println("not start of <TEXT>???");
    		System.exit(-1);
    	}

    	// sometimes its underbars sometimes dashes that delimit the article from author names
    	int author_tag = doc.indexOf("---");
    	if (author_tag == -1)
    		author_tag = doc.indexOf("___");  
    	
    	if ( author_tag > start ) { // stop here and leave the authors but append closing tags
    		end = author_tag;
        	text += doc.substring(start,end);
        	text += "</TEXT>\n";  
    	} else {
        	end = doc.indexOf("</TEXT>") + 8;
        	text += doc.substring(start,end);
    	}
    	
    	return text;
    }

    
    
    /**
     * trims the DOC-segment to a TEXT-segment while keeping the document-ID
     */
    public String docToTextNoID(String doc) {
    	String s = docToText(doc);
    	if (s.length() <= 22) return s;
    	return s.substring(22,s.length());
    }

    
    
    /**
     * Creates a string object from the current content of doc_segment
     */
    public String docToString() {
    	StringBuilder builder = new StringBuilder();
    	for (int i=0; i<doc_pos; i++)
    		builder.append((char)doc_segment[i]);
    	return builder.toString();
    }
    
    
    
    public boolean docIsTypeStory(String doc) {
    	return doc.contains("type=\"story\"");
    }
    
    
    /**
     * In case we find a tag, check if its DOC, 
     * If DOC and we found search-terms then trigger
     * doc_segment dump and do cleanup
     * @param s
     */
    public void readTag(BufferedReader s) {
    	int c=0;
    	int tag_pos = 0;
    	int tag[] = new int[1024];
    	
    	/* read the complete tag */
    	do {
    		c = readChar(s);
    		tag[tag_pos] = c;
    		tag_pos++;
    	} while ((c != '>') && (tag_pos < 1024));
    	
    	/* found <DOC> tag */
    	if ( 	  (tag[0] == 'D')
    			&&(tag[1] == 'O')
    			&&(tag[2] == 'C')) {
    		/* reset the doc_segment */
    		doc_pos = tag_pos+1;
    		doc_segment[0] = '<';
    		System.arraycopy(tag, 0, doc_segment, 1, tag_pos);
    	}
    	
    	/* found closing </DOC> tag AND some keyword(s) matched */
    	if ( 	  (search_term_found || root.hmap.isEmpty())
    			&&(!cleanup)  // when cleaning up we dont print matching articles
    			&&(tag[0] == '/')
    			&&(tag[1] == 'D')
    			&&(tag[2] == 'O')
    			&&(tag[3] == 'C')) {
    		search_term_found = false; // reset
    		    		
    		/* check for duplicate article (DOC-segment) in hash-map */
    		String temp_doc = new String();
    		temp_doc = docToString();
    		String temp_text = docToTextNoID(temp_doc);

    		System.out.println("found new DOC: " + search_term);
    		doc_hmap.put(temp_text, temp_text);
    			
    		appendStringToFile(docToText(temp_doc));
    		doc_pos = 0;
    	}

    	
    	/* found closing </DOC> tag AND NO keyword(s) matched
    	 * but in cleanup mode ... */
    	if ( 	  (!search_term_found)
    			&&(cleanup)
    			&&(tag[0] == '/')
    			&&(tag[1] == 'D')
    			&&(tag[2] == 'O')
    			&&(tag[3] == 'C')) {

    		/* check for duplicate article (DOC-segment) in hash-map */
    		String temp_doc = new String();
    		temp_doc = docToString();
    		if (!docIsTypeStory(temp_doc))
    			return;
    		String temp_text = docToTextNoID(temp_doc);
    		if (doc_hmap.containsKey(temp_text)) {
    			System.out.println("caught duplicate DOC: " + search_term);
    			return;
    		}

    		printDocSegmentToScreen();
    		doc_pos = 0;
    	}
    	
    	/* cleanup found stuff to drop, ok but still we have to
    	 * reset the search flag at the end of each DOC-segment */
    	if ( 	  (search_term_found)
    			&&(cleanup)
    			&&(tag[0] == '/')
    			&&(tag[1] == 'D')
    			&&(tag[2] == 'O')
    			&&(tag[3] == 'C')) {
    		search_term_found = false;
    	}
    }
    
    
    /**
     * Screen dump DOC-segment
     */
    public void printDocSegmentToScreen() {
    	int c;
    	for (c=0; c<doc_pos; c++) {
    		System.out.print((char)doc_segment[c]);
    	}
    	System.out.println();
    }
    
    
    
    /**
     * Read next character and keep track of text that has 
     * been read by copying it to doc_segment
     * @param s
     * @return
     */
    public int readChar(BufferedReader s) {
    	try {
    		int c = s.read();
    		doc_segment[doc_pos] = c;
    		doc_pos++;
    		if (doc_pos >= (MAX_DOC_SIZE-2)) {
    			System.out.println("maximum size of DOC tag exceeded!");
    			System.exit(-1);
    		}
    		return c;
    	} catch (IOException e) {
    		// TODO Auto-generated catch block
    		e.printStackTrace();
    	}
    	return -1;
    }
    
    
    
    
    
    /**
     * Read XML input stream and extract DOC-blocks with search terms in them
     * @param s
     */
    public void searchAndExtract(BufferedReader s, boolean cu) {
    	int c=0;
    	
    	cleanup = cu;
    	if (root.hmap.isEmpty()) {
    		search_term = "extract_all";  // set a default term for extracting everything
    	}
		
    	try {
			if (!s.ready()) return;
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		while(true) {
			/* Read while looking for search-terms in the trie */
			TNode curr_node = root;
			while (true) {
				c = readChar(s);
				if (c < 0) return;
				if (c == '<') {
					readTag(s);
					continue;
				}
				
				c = Tools.toLower(c);
				
				Character jc = new Character((char) c);
				if (curr_node.hmap.containsKey(jc))
					curr_node = curr_node.hmap.get(jc);
				else 
					break;

				if (curr_node.isTerminal && !search_term_found) {
					search_term_found = true;
					search_term = curr_node.term;
				}
			}

		}

    }
}
