package docex;


import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.Charset;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.TreeSet;
import java.util.Vector;
import java.util.Map.Entry;


public class Tools {
	
	/**
	 * A list of ~200 stopwords and very common words in
	 * the english language taken from the snowball project
	 * 
	 * http://snowball.tartarus.org/algorithms/english/stop.txt
	 */
	public static final TreeSet<String> stopWords = new TreeSet<String>(Arrays.asList(
			"i",
			"me",
			"my",
			"myself",
			"we",
			"us",
			"our",
			"ours",
			"ourselves",
			"you",
			"your",
			"yours",
			"yourself",
			"yourselves",
			"he",
			"him",
			"his",
			"himself",
			"she",
			"her",
			"hers",
			"herself",
			"it",
			"its",
			"itself",
			"they",
			"them",
			"their",
			"theirs",
			"themselves",
			"what",
			"which",
			"who",
			"whom",
			"this",
			"that",
			"these",
			"those",
			"am",
			"is",
			"are",
			"was",
			"were",
			"be",
			"been",
			"being",
			"have",
			"has",
			"had",
			"having",
			"do",
			"does",
			"did",
			"doing",
			"would",
			"could",
			"should",
			"ought",
			"will",
			"would",
			"shall",
			"should",
			"can",
			"could",
			"may",
			"might",
			"must",
			"ought",
			"i'm",
			"you're",
			"he's",
			"she's",
			"it's",
			"we're",
			"they're",
			"i've",
			"you've",
			"we've",
			"they've",
			"i'd",
			"you'd",
			"he'd",
			"she'd",
			"we'd",
			"they'd",
			"i'll",
			"you'll",
			"he'll",
			"she'll",
			"we'll",
			"they'll",
			"isn't",
			"aren't",
			"wasn't",
			"weren't",
			"hasn't",
			"haven't",
			"hadn't",
			"doesn't",
			"don't",
			"didn't",
			"won't",
			"wouldn't",
			"shan't",
			"shouldn't",
			"can't",
			"cannot",
			"couldn't",
			"mustn't",
			"let's",
			"that's",
			"who's",
			"what's",
			"here's",
			"there's",
			"when's",
			"where's",
			"why's",
			"how's",
			"a",
			"an",
			"the",
			"and",
			"but",
			"if",
			"or",
			"because",
			"as",
			"until",
			"while",
			"of",
			"at",
			"by",
			"for",
			"with",
			"about",
			"against",
			"between",
			"into",
			"through",
			"during",
			"before",
			"after",
			"above",
			"below",
			"to",
			"from",
			"up",
			"down",
			"in",
			"out",
			"on",
			"off",
			"over",
			"under",
			"again",
			"further",
			"then",
			"once",
			"here",
			"there",
			"when",
			"where",
			"why",
			"how",
			"all",
			"any",
			"both",
			"each",
			"few",
			"more",
			"most",
			"other",
			"some",
			"such",
			"no",
			"nor",
			"not",
			"only",
			"own",
			"same",
			"so",
			"than",
			"too",
			"very",
			"one",
			"every",
			"least",
			"less",
			"many",
			"now",
			"ever",
			"never",
			"say",
			"says",
			"said",
			"also",
			"get",
			"go",
			"goes",
			"just",
			"made",
			"make",
			"put",
			"see",
			"seen",
			"whether",
			"like",
			"well",
			"back",
			"even",
			"still",
			"way",
			"take",
			"since",
			"another",
			"however",
			"two",
			"three",
			"four",
			"five",
			"first",
			"second",
			"new",
			"old",
			"high",
			"long",
			"story", 	// the following stop-words have been added based on frequency observations on the gigaword corpus
			"ap",
			"million",
			"last",
			"utc",
			"billion",
			"year",
			"years"		));

	
	
	/**
	 * Return true if c is in ( [a-z] | [A-Z] )
	 */
    public static boolean isAlpha(int c) {
    	if (c<0x41) return false;
    	if (c>0x5A && c<0x61) return false;
    	if (c>0x7A) return false;
		return true;
    }
	
	
	/**
	 * Return true if c is in ( [a-z] | [A-Z] | [0-9] )
	 */
    public static boolean isAlphaNumeric(int c) {
    	if (c<0x30) return false;
    	if (c>0x39 && c<0x41) return false;
    	if (c>0x5A && c<0x61) return false;
    	if (c>0x7A) return false;
		return true;
    }


    public static int toLower(int c) {
    	/* toLowerCase() */
    	if ((c>0x40)&&(c<0x5B)) {
    		return (c + 0x20); 
    	} else {
    		return c;
    	}
    	
    }
    
		
	/**
	 * JC(A,B) = ((A intersect B) / (A union B)) 
	 * @param first
	 * @param second
	 * @return
	 */
	public static double calcJacquardDistance(TreeSet<String> first, TreeSet<String> second) {
		TreeSet<String> intersect = new TreeSet<String>(first); 
		TreeSet<String> union = new TreeSet<String>(first);
		
		intersect.retainAll(second);
		union.addAll(second);
		
		float i = intersect.size();
		float u = union.size();
		
		return (i/u);
	}
//
//
//	public static boolean potentialStartEndOfWord(int c) {
//		if (		c == ' ' 
//				| 	c == ','
//				| 	c == '\n'
//				|	c == '.'
//				|	c == '('
//				|	c == ')'
//				|	c == '\"'  // all kinds of ""
//				|	c == '\''
////				|	c == '�'
//				|	c == '`'
//				)
//			return true;
//		return false;
//	}
	
	

	public static String readFile(String path, Charset encoding) 
	throws IOException 
	{
		BufferedReader reader = new BufferedReader( new FileReader (path));
		String         line = null;
		StringBuilder  stringBuilder = new StringBuilder();
		String         ls = System.getProperty("line.separator");

		while( ( line = reader.readLine() ) != null ) {
			stringBuilder.append( line );
			stringBuilder.append( ls );
		}

		return stringBuilder.toString();
	}



	public static void writeToFile(String path, String content) {
    	try {
    		FileWriter fw = new FileWriter(path,false);
    		fw.write(content);
    		fw.close();
    	} catch (IOException e) {
    		throw new RuntimeException(e);
    	}
    }


	
	public static void clearFile(String path) {
    	try {
    		FileWriter fw = new FileWriter(path,false);
    		fw.close();
    	} catch (IOException e) {
    		throw new RuntimeException(e);
    	}
    }

	
    
	public static void appendToFile(String path, String content) {
    	try {
    		FileWriter fw = new FileWriter(path,true);
    		fw.write(content);
    		fw.close();
    	} catch (IOException e) {
    		throw new RuntimeException(e);
    	}
    }

	
	
	
	/**
	 * Cut at each <TEXT ....></TEXT> and generate a list of articles 
	 * from the input stream
	 * @param br
	 * @return
	 */
	public static LinkedList<Article> splitStreamToArticles(BufferedReader br) {
		LinkedList<Article> articles = new LinkedList<Article>();
		int start=0, end=0;
		String input = new String();
		StringBuilder builder = new StringBuilder();

		String temp;
		try {
			while (( temp = br.readLine()) != null) {
			    builder.append(temp);
			    builder.append(" ");
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		input = builder.toString();

		// cut up into articles
		do {
			String s = new String();
			if ((end-start)>0) {
				s = input.substring(start+1, end);
				articles.add(new Article(s));
			}
			start = input.indexOf("<TEXT",end);  // include the TEXT tag
			end = input.indexOf("</TEXT>",start)+7;
		} while ((start!=-1)&(end!=-1));

		return articles;
	}


	public static Date getDateFromID(String id) throws ParseException {
		if (id.length()<16)
			id = "APW_ENG_19000101";
		SimpleDateFormat df = new SimpleDateFormat ("yyyy-MM-dd"); 
		Date d = new Date();
		String date = id.substring(8,12)+"-"+id.substring(12,14)+"-"+id.substring(14,16);
		d = df.parse(date); 
		return d;
	}



	/**
	 * Take word frequencies (of an article) normalize them
	 * and multiply with IDF (inverse document frequency) for
	 * every word
	 * @param idf
	 * @param word_frequencies
	 * @return Map with word (term) to TFIDF mapping
	 */
	public static 	HashMap<String, Double> TFIDFfromWordFrequency(
			HashMap<String, Double> idf,
			HashMap<String, Integer> word_frequencies) 
	{	
		HashMap<String, Double> tfidf = new HashMap<String, Double>();
		HashMap<String, Double> normalized_word_frequency = new HashMap<String, Double>();
		int acc,max = 0;

		for (String w : word_frequencies.keySet()) {
			acc = word_frequencies.get(w);
			if (acc > max)
				max = acc; 
		}

		// scale down (normalize)
		for (String w : word_frequencies.keySet()) {
			normalized_word_frequency.put(w, ((double)word_frequencies.get(w))/(double)max);
			//		normalized_word_frequency.put(w, (double)word_frequencies.get(w));
		}

		for (String w : normalized_word_frequency.keySet()) {
			Double d = new Double(normalized_word_frequency.get(w));
			if (idf.containsKey(w)) {
				d = d * idf.get(w);
				tfidf.put(w, d);
			}
		}

		return tfidf;
	}


	
	/**
	 * Calculate similarity between two vectors of "word-frequency-mappings"
	 * by taking their matching (overlapping words between both vectors)
	 * subvectors and calculating their "enclosed angle" (cosine similarity)
	 * The result similarity will be set as a value on the <first> article.
	 * @param x first article/pargraph (TFIDF interface)
	 * @param y second article/paragraph (TFIDF interface)
	 * @return similarity measure [0..1]
	 */
	public static Double similarityFromTFIDFVectors(TFIDF first, TFIDF second) {
		Double sim = (double) 0;
		Double dotproduct,lx,ly;

		HashMap<String,Double> x = first.getTfidf();
		HashMap<String,Double> y = second.getTfidf();

		Vector<Double> vx = new Vector<Double>(); 
		Vector<Double> vy = new Vector<Double>(); 
		
		for (String w : x.keySet()) {
			if (y.containsKey(w)) {
//				if (!x.get(w).equals(y.get(w)))
//					System.out.println(w+" <> "+x.get(w)+" "+y.get(w));
//				else
//					System.out.println(w+" == "+x.get(w)+" "+y.get(w));
				vx.add(x.get(w));
				vy.add(y.get(w));
			}
		}

//		System.out.println("similarityFromTFIDFVectors("+x.size()+","+y.size()+") with "+vx.size()+" common words");

		if (vx.size() == 0)
			return ((double)0);  // not even one word in common
		
		dotproduct = (double)0;
		for (int i=0; i < vx.size(); i++) {
			dotproduct += vx.get(i) * vy.get(i);
		}
		first.setDotproduct_similarity(dotproduct/200);
		
		lx = (double)0;
		for (Entry<String,Double> e : x.entrySet()) {
			lx += e.getValue() * e.getValue();
		}
		lx = Math.sqrt(lx);
		
		ly = (double)0;
		for (Entry<String,Double> e : y.entrySet()) {
			ly += e.getValue() * e.getValue();
		}
		ly = Math.sqrt(ly);
		
//		System.out.println("dotproduct is "+dotproduct);
		
		sim = Math.acos(dotproduct / (lx * ly));
		
		sim = 1 - (sim / (Math.PI/2));  // scale to [0..1]
		
		if (sim.isNaN())
			sim = (double)1;
		
		first.setCosine_similarity(sim);
		
		return sim;
	}
	
	
	


	/**
	 * Checks if all words appearing in newset have been seen in the oldset already.
	 * @param newset 
	 * @param oldset
	 * @return TRUE if <newset> is a subset of <oldset>
	 */
	public static boolean isSubsetOf(TFIDF newset, TFIDF oldset) {
		return newset.getWord_frequencies().keySet().containsAll(oldset.getWord_frequencies().keySet());
	}
	
	
	
}



